@@ -16,10 +16,11 @@ import warnings
 
 
 class CloudConfig(object):
-    def __init__(self, name, region, config):
+    def __init__(self, name, region, config, prefer_ipv6=False):
         self.name = name
         self.region = region
         self.config = config
+        self._prefer_ipv6 = prefer_ipv6
 
     def __getattr__(self, key):
         """Return arbitrary attributes."""
@@ -73,11 +74,12 @@ class CloudConfig(object):
     def get_auth_args(self):
         return self.config['auth']
 
-    def get_endpoint_type(self, service_type=None):
+    def get_interface(self, service_type=None):
+        interface = self.config.get('interface')
         if not service_type:
-            return self.config['endpoint_type']
-        key = '{service_type}_endpoint_type'.format(service_type=service_type)
-        return self.config.get(key, self.config['endpoint_type'])
+            return interface
+        key = '{service_type}_interface'.format(service_type=service_type)
+        return self.config.get(key, interface)
 
     def get_region_name(self, service_type=None):
         if not service_type:
@@ -95,4 +97,8 @@ class CloudConfig(object):
 
     def get_service_name(self, service_type):
         key = '{service_type}_service_name'.format(service_type=service_type)
-        return self.config.get(key, service_type)
+        return self.config.get(key, None)
+
+    @property
+    def prefer_ipv6(self):
+        return self._prefer_ipv6
